"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Registry = exports.DomainSchema = void 0;
const schema_registry_sdk_1 = require("@alicloud/schema-registry-sdk");
const confluent_schema_registry_1 = require("@kafkajs/confluent-schema-registry");
const glue_schema_registry_1 = require("@meinestadt.de/glue-schema-registry");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const misc_1 = require("../misc");
class DomainSchema {
    schemaValue;
    schema;
    registry;
    _registryId;
    constructor(schema, disableSchemaRegistry = false) {
        if ('string' !== typeof schema) {
            schema = JSON.stringify(schema);
        }
        this.schemaValue = schema;
        if ('boolean' === typeof disableSchemaRegistry) {
            if (!disableSchemaRegistry) {
                this.registry = Registry.isEnabled ? Registry.instance : undefined;
            }
        }
        else {
            this.registry = disableSchemaRegistry || (Registry.isEnabled ? Registry.instance : undefined);
        }
        if (!this.registry && schema) {
            this.schema = (0, misc_1.stringToAvroSchema)(schema);
        }
    }
    async encode(value) {
        try {
            if (this.schema) {
                return this.schema.toBuffer(value);
            }
            if (undefined === this.registryId) {
                if (!this.isEnabled) {
                    (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'Schema is not initialized');
                }
                this._registryId = await this.registry.register(this.schemaValue);
            }
            return await this.registry.encode(this.registryId, value);
        }
        catch (error) {
            (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.INVALID_MESSAGE, { registryId: this.registryId });
        }
    }
    async decode(value) {
        try {
            if (this.schema) {
                return this.schema.fromBuffer(value);
            }
            return await this.registry.decode(value);
        }
        catch (error) {
            (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.INVALID_MESSAGE);
        }
    }
    get isEnabled() {
        return this.schemaValue ? true : false;
    }
    get isSchemaRegistryEnabled() {
        return this.registry ? true : false;
    }
    get registryId() {
        return this._registryId;
    }
}
exports.DomainSchema = DomainSchema;
class Registry {
    static self;
    url;
    overrides;
    schemaRegistry;
    alicloudSchemaRegistry;
    glueSchemaRegistry;
    constructor(url, overrides) {
        this.url = url;
        this.overrides = overrides;
        if (overrides?.accessKeyId && overrides?.accessKeySecret && overrides?.groupId) {
            this.alicloudSchemaRegistry = new schema_registry_sdk_1.SchemaRegistry({ endpoint: url, ...overrides });
        }
        else if (overrides?.awsRegion) {
            this.glueSchemaRegistry = new glue_schema_registry_1.GlueSchemaRegistry(url, { region: overrides?.awsRegion });
        }
        else {
            this.schemaRegistry = new confluent_schema_registry_1.SchemaRegistry({ host: url, ...overrides });
        }
    }
    static init(url, overrides) {
        this.self = new Registry(url, overrides);
        return this.self;
    }
    static term() {
        this.self = undefined;
    }
    clone() {
        return new Registry(this.url, this.overrides);
    }
    async register(schema) {
        if (this.alicloudSchemaRegistry) {
            return (await this.alicloudSchemaRegistry.register({ schema, type: schema_registry_sdk_1.SchemaType.AVRO }))?.id;
        }
        if (this.glueSchemaRegistry) {
            const afterCleanUpSchema = (0, misc_1.stringToJson)(JSON.stringify(schema));
            const tempSchema = JSON.stringify(afterCleanUpSchema);
            // create a new schema
            // throws an error if the schema already exists
            try {
                await this.glueSchemaRegistry.createSchema({
                    type: glue_schema_registry_1.SchemaType.AVRO,
                    schemaName: afterCleanUpSchema['name'],
                    compatibility: glue_schema_registry_1.SchemaCompatibilityType.BACKWARD,
                    schema: tempSchema
                });
            }
            catch (error) {
                /// just ignore the error
                // console.log('already exist:', error)
            }
            return await this.glueSchemaRegistry.register({ schema: tempSchema, schemaName: afterCleanUpSchema['name'], type: glue_schema_registry_1.SchemaType.AVRO });
        }
        return (await this.schemaRegistry.register({ schema, type: confluent_schema_registry_1.SchemaType.AVRO }))?.id;
    }
    async encode(id, payload) {
        if (this.alicloudSchemaRegistry) {
            return await this.alicloudSchemaRegistry.encode(id, payload);
        }
        if (this.glueSchemaRegistry) {
            return await this.glueSchemaRegistry.encode(id, payload);
        }
        return this.schemaRegistry.encode(id, payload);
    }
    async decode(payload) {
        if (this.alicloudSchemaRegistry) {
            return this.alicloudSchemaRegistry.decode(payload);
        }
        if (this.glueSchemaRegistry) {
            const analyzeMessage = await this.glueSchemaRegistry.analyzeMessage(payload);
            return await this.glueSchemaRegistry.decode(payload, (0, misc_1.stringToAvroSchema)(analyzeMessage.schema.SchemaDefinition));
        }
        return this.schemaRegistry.decode(payload);
    }
    static get instance() {
        return this.self || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'Registry initialization is required');
    }
    static get isEnabled() {
        return undefined !== this.self;
    }
}
exports.Registry = Registry;
//# sourceMappingURL=domain-schema.js.map