"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorHandler = void 0;
const graphql_1 = require("graphql");
const graphql_tag_1 = require("graphql-tag");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_utils_1 = require("cm-libs-utils");
const locale_1 = require("../locale");
const intercall_1 = require("../intercall");
class ErrorHandler {
    static self;
    errorUrl;
    serviceAccessInfo;
    constructor(url, serviceAccessInfo) {
        this.errorUrl = url;
        this.serviceAccessInfo = serviceAccessInfo || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.PARAMETER_MISSING, 'Service id token is missing');
    }
    static init(url, serviceAccessInfo) {
        this.self = new ErrorHandler(url, serviceAccessInfo);
    }
    static async translate(error, statusCode, localeCode) {
        const { code, message } = (0, cm_libs_context_aware_error_1.describeError)(error);
        // Override status code by error specified status code
        if (error?.statusCode) {
            statusCode = (0, cm_libs_utils_1.checkFormat)((0, cm_libs_utils_1.allowNullOrEmpty)(cm_libs_utils_1.checkNumber), error?.statusCode);
        }
        // Assign specific status code, otherwise respect the status code that caller specified
        if (!statusCode || 200 === statusCode) {
            // Override status code that based on the error code
            /* istanbul ignore next */
            switch (code) {
                case cm_libs_context_aware_error_1.errors.UNAUTHORISED:
                case cm_libs_context_aware_error_1.errors.EXPIRED:
                    statusCode = 401;
                    break;
                case cm_libs_context_aware_error_1.errors.FORBIDDEN:
                    statusCode = 403;
                    break;
                case 'TooManyRequests':
                    statusCode = 429;
                    break;
                default:
                    statusCode = 500;
            }
        }
        try {
            const translatedErrorMessage = this.isEnabled ? await this.instance.translateErrorMessage(code, localeCode) : undefined;
            return {
                statusCode,
                code,
                message: translatedErrorMessage || this.fallbackErrorMessage(message)
            };
        }
        catch (error) {
            return {
                statusCode,
                code,
                message: this.fallbackErrorMessage(message)
            };
        }
    }
    static fallbackErrorMessage(message) {
        // Get the last line of multi-line message, if any
        if (message?.includes('\n')) {
            const lines = message.split('\n');
            return lines?.[lines.length - 1]?.trim() || lines?.[lines.length - 2]?.trim();
        }
        return message;
    }
    async translateErrorMessage(code, localeCode = locale_1.LocaleCode.EN_US) {
        const query = (0, graphql_tag_1.gql) `
            query PaidChain {
                readErrorMessage(where: { code_localeCode: { code: "${code}", localeCode: "${localeCode}" } }) {
                    reason
                }
            }
        `;
        const body = { query: (0, graphql_1.print)(query) };
        try {
            const { reason } = await (0, intercall_1.callGraphQl)(this.errorUrl, body, 'readErrorMessage', this.serviceAccessInfo ? { authorization: this.serviceAccessInfo?.token } : undefined);
            return reason;
        }
        catch (error) {
            // Ignore
        }
    }
    static get instance() {
        return this.self || (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.NOT_INITIALIZED, 'ErrorHandler initialization is required');
    }
    static get isEnabled() {
        return undefined !== this.self;
    }
}
exports.ErrorHandler = ErrorHandler;
//# sourceMappingURL=error.js.map