"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ModelFactory = exports.GraphQLHelper = void 0;
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const graphql_1 = require("graphql");
const graphql_parse_resolve_info_1 = require("graphql-parse-resolve-info");
const lodash_1 = require("lodash");
const node_fs_1 = __importDefault(require("node:fs"));
const node_path_1 = __importDefault(require("node:path"));
const config_base_1 = require("../config-base");
class GraphQLHelper {
    static #mapping;
    static setMap(jsonObject) {
        this.#mapping = jsonObject;
    }
    static fieldsSelection(info) {
        const parsedResolveInfoFragment = (0, graphql_parse_resolve_info_1.parseResolveInfo)(info);
        const { fields } = (0, graphql_parse_resolve_info_1.simplifyParsedResolveInfoFragmentWithType)(parsedResolveInfoFragment, info.returnType);
        return this.#resolvingKeys(fields, true);
    }
    static itemsFieldsSelection(info, entity) {
        const parsedResolveInfoFragment = (0, graphql_parse_resolve_info_1.parseResolveInfo)(info);
        const { fields } = (0, graphql_parse_resolve_info_1.simplifyParsedResolveInfoFragmentWithType)(parsedResolveInfoFragment, info.returnType);
        let selection;
        for (const [key, value] of Object.entries(fields)) {
            if ((0, lodash_1.isEqual)(value['fieldsByTypeName'], {})) {
                selection ??= {};
                selection[key] = true;
            }
            else {
                const found = value['fieldsByTypeName'][entity];
                if (found) {
                    selection = this.#resolvingKeys(found, true);
                }
            }
        }
        return selection;
    }
    static requestQueryDSLAtResolverLevel(info) {
        const selection = this.requestSelectAtResolverLevel(info, false);
        let result = JSON.stringify(selection);
        result = result.replaceAll(':true', ' ').replaceAll('"', '').replaceAll(':', '').replaceAll(',', '');
        return result;
    }
    static requestSelectAtResolverLevel(info, withSelect = false) {
        const parsedResolveInfoFragment = (0, graphql_parse_resolve_info_1.parseResolveInfo)(info);
        const { fields } = (0, graphql_parse_resolve_info_1.simplifyParsedResolveInfoFragmentWithType)(parsedResolveInfoFragment, info.returnType);
        return this.#resolvingKeys(fields, withSelect);
    }
    static #resolvingKeys(fields, withSelect = false) {
        let selection;
        for (const [key, value] of Object.entries(fields)) {
            if ((0, lodash_1.isEqual)(value['fieldsByTypeName'], {})) {
                selection ??= {};
                selection[key] = true;
            }
            else {
                const found = value['fieldsByTypeName'][this.#mapping[key]];
                if (found) {
                    if (withSelect)
                        selection[key] = { select: this.#resolvingKeys(found, withSelect) };
                    else
                        selection[key] = this.#resolvingKeys(found, withSelect);
                }
            }
        }
        return selection;
    }
    static getResolvers(obj) {
        const resolvers = [];
        const keys = Object.keys(obj);
        for (let i = 0, l = keys.length; i < l; ++i) {
            if (obj[keys[i]]['Query'] || obj[keys[i]]['Mutation']) {
                resolvers.push(obj[keys[i]]);
            }
        }
        return resolvers;
    }
    static loadSchemaFiles(finalGraphql, pathString, isExternal = false) {
        node_fs_1.default.readdirSync(pathString).forEach((file) => {
            if (node_path_1.default.extname(file) === '.graphql') {
                const entity = (0, lodash_1.upperFirst)((0, lodash_1.camelCase)(file.replace('.schema.graphql', '')));
                if (!isExternal) {
                    finalGraphql.dynamic += `type ${entity}Paginated {
                items: [${entity}]
                properties: Properties
                }\n`;
                }
                finalGraphql.typeDefs += node_fs_1.default.readFileSync(node_path_1.default.join(pathString, file), 'utf8').concat(`\n`);
            }
        });
    }
}
exports.GraphQLHelper = GraphQLHelper;
class ModelFactory {
    listOfApis;
    type;
    listToResolve;
    queryOperationStartsWith;
    queryOperationBrowseStartsWith;
    transformArgs;
    QUERY_OPERATION_START_WITH = config_base_1.ConfigBase.get('QUERY_OPERATION_START_WITH', (x) => {
        x = x ? JSON.parse(x) : ['browse', 'read', 'onBoardBrowse', 'onBoardRead'];
        if (!Array.isArray(x))
            throw new Error('invalid array of QUERY_OPERATION_START_WITH');
        return x;
    });
    QUERY_OPERATION_BROWSE_START_WITH = config_base_1.ConfigBase.get('QUERY_OPERATION_BROWSE_START_WITH', (x) => {
        x = x ? JSON.parse(x) : ['browse', 'onBoardBrowse'];
        if (!Array.isArray(x))
            throw new Error('invalid array of QUERY_OPERATION_BROWSE_START_WITH');
        return x;
    });
    query = {};
    mutation = {};
    constructor(listOfApis, type, listToResolve = [], queryOperationStartsWith = [], queryOperationBrowseStartsWith = [], transformArgs) {
        this.listOfApis = listOfApis;
        this.type = type;
        this.listToResolve = listToResolve;
        this.queryOperationStartsWith = queryOperationStartsWith;
        this.queryOperationBrowseStartsWith = queryOperationBrowseStartsWith;
        this.transformArgs = transformArgs;
        this.queryOperationStartsWith = (0, lodash_1.union)(this.QUERY_OPERATION_START_WITH, queryOperationStartsWith);
        this.queryOperationBrowseStartsWith = (0, lodash_1.union)(this.QUERY_OPERATION_BROWSE_START_WITH, queryOperationBrowseStartsWith);
        if (listOfApis)
            this.#processOperation(this.listOfApis);
    }
    async resolve(fn, args, context, resolveInfo, isBrowse = false) {
        try {
            if (this.transformArgs) {
                args = this.transformArgs(args, context, resolveInfo, isBrowse);
                return await fn(args, context.accessInfo);
            }
            if (isBrowse)
                args.select = GraphQLHelper.itemsFieldsSelection(resolveInfo, this.type);
            else
                args.select = GraphQLHelper.fieldsSelection(resolveInfo);
            return await fn(args, context.accessInfo);
        }
        catch (error) {
            const { code, message } = (0, cm_libs_context_aware_error_1.describeError)(error);
            throw new graphql_1.GraphQLError(message, { originalError: error, extensions: { code, originalError: error } });
        }
    }
    #processOperation(allComponents) {
        const functions = Object.values(allComponents);
        functions.forEach((component) => {
            if (component.constructor.name === 'AsyncFunction') {
                if (this.#checkName(this.queryOperationStartsWith, component.name)) {
                    this.query[component.name] = (_data, args, context, resolveInfo) => this.resolve(component, args, context, resolveInfo, this.#checkName(this.queryOperationBrowseStartsWith, component.name));
                }
                else {
                    this.mutation[component.name] = (_data, args, context, resolveInfo) => this.resolve(component, args, context, resolveInfo);
                }
            }
        });
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => `${this.type} has ${functions.length} functions loaded`);
    }
    #checkName(listOfQueryName, functionName) {
        for (const iterator of listOfQueryName) {
            if (functionName.startsWith(iterator)) {
                return true;
            }
        }
        return false;
    }
}
exports.ModelFactory = ModelFactory;
//# sourceMappingURL=graphql.js.map