"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseModelMiddleware = exports.OmitUpdateInput = exports.OmitCreateInput = exports.RecordStatus = void 0;
const lodash_1 = __importDefault(require("lodash"));
const async_hooks_1 = require("async_hooks");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cm_libs_utils_1 = require("cm-libs-utils");
const auth_1 = require("../auth");
const model_middleware_role_ability_log_1 = require("./model-middleware-role-ability-log");
const app_seed_1 = require("../../app-seed");
var RecordStatus;
(function (RecordStatus) {
    RecordStatus["ACTIVE"] = "ACTIVE";
    RecordStatus["INACTIVE"] = "INACTIVE";
})(RecordStatus || (exports.RecordStatus = RecordStatus = {}));
exports.OmitCreateInput = { createdBy: 0, createdByName: '' };
exports.OmitUpdateInput = { updatedBy: 0, updatedByName: '' };
class BaseModelMiddleware {
    client;
    errorHandler;
    asyncLocalStorage;
    constructor(client, errorHandler) {
        this.asyncLocalStorage = new async_hooks_1.AsyncLocalStorage();
        this.errorHandler = errorHandler;
        client['$use'](async (params, next) => this.process(params, next));
        if (model_middleware_role_ability_log_1.BaseModelMiddlewareRoleAbilityLog.isEnabled) {
            client['$use'](async (params, next) => model_middleware_role_ability_log_1.BaseModelMiddlewareRoleAbilityLog.instance.process(params, next, this.asyncLocalStorage));
        }
        this.client = client;
    }
    async run(context, promise) {
        return this.asyncLocalStorage.run(context, async () => {
            return await promise();
        });
    }
    async process(params, next) {
        const start = (0, cm_libs_utils_1.timeNow)();
        const context = this.asyncLocalStorage.getStore();
        switch (params.action) {
            case 'findUnique':
                this.packFindUniqueArgs(params, context);
                break;
            case 'findMany':
                this.packFindManyArgs(params, context);
                break;
            case 'create':
                this.packCreateArgs(params, context);
                break;
            case 'createMany':
                this.packCreateManyArgs(params, context);
                break;
            case 'update':
                this.packUpdateArgs(params, context);
                break;
            case 'updateMany':
                this.packUpdateManyArgs(params, context);
                break;
            case 'upsert':
                this.packUpsertArgs(params, context);
                break;
            case 'delete':
                this.packDeleteArgs(params, context);
                break;
            case 'deleteMany':
                this.packDeleteManyArgs(params, context);
                break;
        }
        if (app_seed_1.devMode) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DETAIL, () => `Run ${params.model}.${params.action}: ${JSON.stringify(params, null, 2)}`);
        }
        try {
            const result = await next(params);
            (0, cm_libs_context_aware_logger_1.log)(app_seed_1.devMode ? cm_libs_context_aware_logger_1.Level.DETAIL : cm_libs_context_aware_logger_1.Level.DEBUG, () => `Run ${params.model}.${params.action} affected ${affectedCount(params.action, result)} record(s) took ${(0, cm_libs_utils_1.timeElapsed)(start)} ms`);
            return result;
        }
        catch (error) {
            this.errorHandler ? this.errorHandler(error, params, context) : (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.DATABASE_ERROR, { params, context });
        }
    }
    packFindUniqueArgs(params, context) {
        this.restrictInclude(params.args, params.action, params.model);
        if (!context.disableRestriction) {
            const roleModel = this.restrictSelect(params.args, params.action, params.model, context);
            this.restrictWhereUnique(params.args, roleModel?.findUnique?.where, context);
        }
    }
    packFindManyArgs(params, context) {
        this.restrictInclude(params.args, params.action, params.model);
        if (!context.disableRestriction) {
            const roleModel = this.restrictSelect(params.args, params.action, params.model, context);
            this.restrictWhere(params.args, roleModel?.findMany?.where, context);
        }
    }
    packCreateArgs(params, context) {
        const roleModel = context.disableRestriction ? undefined : this.restrictSelect(params.args, params.action, params.model, context);
        const schema = context.schemas?.schemaCreateInput;
        const requestedBy = context.accessInfo.requestedBy;
        const requestedByName = context.accessInfo.requestedByName;
        params.args.data = {
            ...validatedValues(params.args.data, schema),
            createdBy: requestedBy,
            createdByName: requestedByName
        };
        this.restrictCreate(params.args.data, 'create', roleModel, context);
    }
    packCreateManyArgs(params, context) {
        const roleModel = context.disableRestriction ? undefined : this.getRoleModel(params.action, params.model, context.accessInfo);
        const schema = context.schemas?.schemaCreateInput;
        const requestedBy = context.accessInfo.requestedBy;
        const requestedByName = context.accessInfo.requestedByName;
        params.args.data = params.args.data?.map((item) => {
            const data = {
                ...validatedValues(item, schema),
                createdBy: requestedBy,
                createdByName: requestedByName
            };
            this.restrictCreate(data, 'createMany', roleModel, context);
            return data;
        });
    }
    packUpdateArgs(params, context) {
        const roleModel = context.disableRestriction ? undefined : this.restrictSelect(params.args, params.action, params.model, context);
        const schema = context.schemas?.schemaUpdateInput;
        const requestedBy = context.accessInfo.requestedBy;
        const requestedByName = context.accessInfo.requestedByName;
        params.args.data = {
            ...validatedValues(params.args.data, schema),
            updatedBy: requestedBy,
            updatedByName: requestedByName
        };
        this.restrictWhereUnique(params.args, roleModel?.update?.where, context);
        this.restrictFields(params.args.data, roleModel?.update?.data, context);
    }
    packUpdateManyArgs(params, context) {
        const roleModel = context.disableRestriction ? undefined : this.getRoleModel(params.action, params.model, context.accessInfo);
        this.restrictWhereUnique(params.args, roleModel?.updateMany?.where, context);
        const schema = context.schemas?.schemaUpdateInput;
        const requestedBy = context.accessInfo.requestedBy;
        const requestedByName = context.accessInfo.requestedByName;
        if (Array.isArray(params.args.data)) {
            params.args.data = params.args.data?.map((item) => {
                const data = {
                    ...validatedValues(item, schema),
                    updatedBy: requestedBy,
                    updatedByName: requestedByName
                };
                this.restrictFields(data, roleModel?.updateMany?.data, context);
                return data;
            });
        }
        else {
            params.args.data = {
                ...validatedValues(params.args.data, schema),
                updatedBy: requestedBy,
                updatedByName: requestedByName
            };
            this.restrictFields(params.args.data, roleModel?.updateMany?.data, context);
        }
    }
    packUpsertArgs(params, context) {
        const roleModel = context.disableRestriction ? undefined : this.restrictSelect(params.args, 'update', params.model, context);
        this.restrictWhereUnique(params.args, roleModel?.update?.where, context);
        const schemaCreateInput = context.schemas?.schemaCreateInput;
        const schemaUpdateInput = context.schemas?.schemaUpdateInput;
        const requestedBy = context.accessInfo.requestedBy;
        const requestedByName = context.accessInfo.requestedByName;
        params.args.update = {
            ...validatedValues(params.args.update, schemaUpdateInput),
            updatedBy: requestedBy,
            updatedByName: requestedByName
        };
        this.restrictFields(params.args.update, roleModel?.update?.data, context);
        params.args.create = {
            ...validatedValues(params.args.create, schemaCreateInput),
            createdBy: requestedBy,
            createdByName: requestedByName
        };
        const roleModelForCreate = context.disableRestriction ? undefined : this.getRoleModel('create', params.model, context.accessInfo);
        this.restrictCreate(params.args.create, 'create', roleModelForCreate, context);
    }
    packDeleteArgs(params, context) {
        const roleModel = context.disableRestriction ? undefined : this.restrictSelect(params.args, params.action, params.model, context);
        this.restrictWhereUnique(params.args, roleModel?.delete?.where, context);
    }
    packDeleteManyArgs(params, context) {
        if (!context.disableRestriction) {
            const roleModel = this.getRoleModel(params.action, params.model, context.accessInfo);
            this.restrictWhere(params.args, roleModel?.deleteMany?.where, context);
        }
    }
    restrictCreate(data, action, roleModel, context) {
        if (data) {
            for (const key of Object.keys(data)) {
                const item = data[key];
                if ('object' === typeof item) {
                    const subItem = item.connectOrCreate || item;
                    const requestedBy = context.accessInfo.requestedBy;
                    const requestedByName = context.accessInfo.requestedByName;
                    if (undefined !== subItem?.create) {
                        if (Array.isArray(subItem.create)) {
                            subItem.create = subItem.create.map((value) => {
                                return { ...value, createdBy: requestedBy, createdByName: requestedByName };
                            });
                        }
                        else {
                            subItem.create = { ...subItem.create, createdBy: requestedBy, createdByName: requestedByName };
                        }
                        if (roleModel) {
                            const subItemModel = this.selectableModel(key, roleModel);
                            const subRoleModel = this.getRoleModel(action, subItemModel.model, context.accessInfo);
                            this.restrictFields(subItem.create, subRoleModel?.[action]?.data, context);
                            if (item.connectOrCreate) {
                                const subRoleModel = this.getRoleModel('findUnique', subItemModel.model, context.accessInfo);
                                this.restrictWhereUnique(subItem, subRoleModel?.findUnique?.where, context);
                            }
                        }
                    }
                }
            }
            this.restrictFields(data, roleModel?.[action]?.data, context);
        }
    }
    restrictSelect(args, action, model, context) {
        const roleModel = this.getRoleModel(action, model, context.accessInfo);
        if (args.select && roleModel) {
            for (const key of Object.keys(args.select)) {
                const item = args.select[key];
                if ('object' === typeof item && item.select) {
                    const itemModel = this.selectableModel(key, roleModel);
                    const itemRoleModel = this.restrictSelect(item, action, itemModel.model, context);
                    if (true === itemModel.oneToOne) {
                        this.restrictWhereUnique(args, itemRoleModel?.findUnique?.where, context, key);
                    }
                    else {
                        this.restrictWhere(item, itemRoleModel?.findMany?.where, context);
                    }
                }
            }
        }
        return roleModel;
    }
    restrictInclude(args, action, model) {
        if (args.include) {
            const context = this.asyncLocalStorage.getStore();
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.FORBIDDEN, 'Forbidden to access model via include method', {
                action,
                model,
                rolesMetadata: context?.accessInfo?.auth?.rolesMetadata,
                userId: context?.accessInfo?.requestedBy,
                disableRestriction: context?.disableRestriction
            });
        }
    }
    restrictWhere(args, where, context) {
        if (where) {
            // Render the placeholder to values
            where = (0, auth_1.renderValues)(where, { accessInfo: context.accessInfo, args, where, ref: context.accessInfo.ref });
            args.where ??= {};
            args.where.AND ??= [];
            if (Array.isArray(where)) {
                args.where.AND = args.where.AND.concat(where);
            }
            else {
                args.where.AND.push(where);
            }
        }
    }
    restrictWhereUnique(args, where, context, model) {
        if (where) {
            // Render the placeholder to values
            where = (0, auth_1.renderValues)(where, { accessInfo: context.accessInfo, args, where, ref: context.accessInfo.ref });
            args.where ??= {};
            for (const key of Object.keys(where)) {
                if (undefined !== args.where[key] && !lodash_1.default.isEqual(args.where[key], where[key])) {
                    (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.FORBIDDEN, 'Forbidden to access model on restricted record', {
                        model,
                        field: key,
                        expectedValue: JSON.stringify(where[key]),
                        rolesMetadata: context.accessInfo?.auth?.rolesMetadata,
                        userId: context.accessInfo?.requestedBy,
                        disableRestriction: context.disableRestriction
                    });
                }
            }
            if (!model) {
                args.where = { ...args.where, ...where };
            }
            else {
                if (args.where[model]) {
                    args.where[model].is ??= {};
                    args.where[model].is = {
                        ...args.where[model].is,
                        ...where
                    };
                }
                else {
                    args.where = { ...args.where, [model]: where };
                }
            }
        }
    }
    restrictFields(fields, data, context) {
        if (data) {
            const objects = Array.isArray(fields) ? fields : [fields];
            for (const object of objects) {
                for (const key of Object.keys(object)) {
                    if (undefined !== data[key]) {
                        // Render the placeholder to values
                        const value = (0, auth_1.renderValues)(data[key], { accessInfo: context.accessInfo, fields, data, ref: context.accessInfo.ref });
                        if (!lodash_1.default.isEqual(fields[key], value)) {
                            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.FORBIDDEN, 'Forbidden to access model on restricted field', {
                                field: key,
                                expectedValue: JSON.stringify(value),
                                rolesMetadata: context.accessInfo?.auth?.rolesMetadata,
                                userId: context.accessInfo?.requestedBy,
                                disableRestriction: context.disableRestriction
                            });
                        }
                    }
                }
            }
        }
    }
    getRoleModel(action, modelName, accessInfo) {
        if (Array.isArray(accessInfo.auth?.rolesMetadata)) {
            let restriction = true;
            for (const roleMetadata of accessInfo.auth.rolesMetadata) {
                const model = roleMetadata.models?.[modelName];
                if (model) {
                    if (action) {
                        if (model[action]) {
                            return model;
                        }
                        if (model['*']) {
                            restriction = false;
                            break;
                        }
                        const context = this.asyncLocalStorage.getStore();
                        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.FORBIDDEN, 'Forbidden to operate model', {
                            action,
                            model: modelName,
                            rolesMetadata: accessInfo.auth.rolesMetadata,
                            userId: accessInfo.requestedBy,
                            disableRestriction: context?.disableRestriction
                        });
                    }
                    return model;
                }
                // Disable restriction, but give priority to defined restriction if any
                if (roleMetadata.models?.['*']) {
                    const model = roleMetadata.models['*'];
                    if (action && model[action]) {
                        return model;
                    }
                    restriction = false;
                }
            }
            if (!restriction) {
                return undefined;
            }
        }
        const context = this.asyncLocalStorage.getStore();
        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.FORBIDDEN, 'Forbidden to access model', {
            action,
            model: modelName,
            rolesMetadata: accessInfo.auth?.rolesMetadata,
            userId: accessInfo.requestedBy,
            disableRestriction: context?.disableRestriction
        });
    }
    selectableModel(model, roleModel) {
        const selected = roleModel.select?.[model];
        if (undefined !== selected && 'object' === typeof selected) {
            return selected;
        }
        const context = this.asyncLocalStorage.getStore();
        (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.errors.FORBIDDEN, 'Forbidden to access foreign model', {
            model,
            rolesMetadata: context?.accessInfo?.auth?.rolesMetadata,
            userId: context?.accessInfo?.requestedBy,
            disableRestriction: context?.disableRestriction
        });
    }
}
exports.BaseModelMiddleware = BaseModelMiddleware;
function affectedCount(action, result) {
    let count;
    switch (action) {
        case 'findUnique':
        case 'update':
            count = result ? 1 : 0;
            break;
        case 'findMany':
        case 'queryRaw':
            count = result?.length;
            break;
        case 'count':
        case 'executeRaw':
            count = Number(result);
            break;
        case 'create':
        case 'upsert':
            count = 1;
            break;
        case 'createMany':
        case 'updateMany':
            count = result?.count;
    }
    return count || 0;
}
function validatedValues(data, schema) {
    const { error, value } = schema ? schema.validate(data, { stripUnknown: true }) : { error: undefined, value: data };
    if (error) {
        (0, cm_libs_context_aware_error_1.throwFromError)(error, cm_libs_context_aware_error_1.errors.VALIDATION_ERROR, { data });
    }
    return value;
}
//# sourceMappingURL=model-middleware.js.map