"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseRESTDataSource = void 0;
const datasource_rest_1 = require("@apollo/datasource-rest");
const utils_keyvaluecache_1 = require("@apollo/utils.keyvaluecache");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const correlation_id_1 = __importDefault(require("correlation-id"));
const graphql_1 = require("graphql");
const lodash_1 = __importDefault(require("lodash"));
const nanoid_1 = require("nanoid");
const misc_1 = require("../misc");
const defaultCacheSetting = {
    max: 500,
    // for use with tracking overall storage size
    maxSize: 5000,
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    sizeCalculation: (value, key) => {
        return 1;
    },
    // 5 seconds (in milliseconds)
    ttl: 5 * 1000
};
class BaseRESTDataSource extends datasource_rest_1.RESTDataSource {
    token;
    cache;
    baseURL = '';
    constructor(options) {
        super(options);
        this.token = options?.token;
        this.cache = options?.cache || new utils_keyvaluecache_1.InMemoryLRUCache(defaultCacheSetting);
    }
    willSendRequest(_path, request) {
        if (!request?.headers['authorization'])
            request.headers['authorization'] = this.token;
        request.headers['x-correlation-id'] = correlation_id_1.default.getId() || (0, nanoid_1.nanoid)();
    }
    /* eslint-disable @typescript-eslint/no-unused-vars */
    async resolveURL(path, _request) {
        if (!this.baseURL) {
            this.baseURL = path;
        }
        if (!this.baseURL.endsWith('/')) {
            this.baseURL += '/';
        }
        if (path.startsWith('/')) {
            path = path.substring(1);
        }
        return new URL(path, this.baseURL);
    }
    async invokeGraphqlWithCache(functionName, body, accessInfo, allowCache = false, ttlSeconds = 5) {
        if (allowCache) {
            const key = JSON.stringify({ functionName, query: body });
            const value = await this.cache.get(key);
            if (value) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => 'Cached value');
                return value;
            }
            else {
                const data = await this.invokeGraphql(functionName, body, accessInfo);
                await this.cache.set(key, data, { ttl: ttlSeconds });
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => 'New value');
                return data;
            }
        }
        else {
            return this.invokeGraphql(functionName, body, accessInfo);
        }
    }
    async invokeGraphql(functionName, body, accessInfo) {
        let data = null;
        try {
            const headers = {
                authorization: accessInfo?.token
            };
            if (accessInfo?.headers?.roleType) {
                headers['x-role-type'] = accessInfo.headers.roleType;
            }
            const { data: newData, errors } = await this.post('/graphql', {
                headers,
                body
            });
            if (newData) {
                data = newData[functionName];
            }
            if (lodash_1.default.isNil(data) && errors?.length > 0)
                throw new graphql_1.GraphQLError(errors[0].message, { path: errors[0].path, extensions: errors[0].extensions });
        }
        catch (error) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.EXCEPTION, () => `invokeGraphql error:${JSON.stringify(error, null, 2)}`);
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.ErrorCodes.IntercallError, error?.extensions?.response?.body?.errors?.[0].message || error.message);
        }
        misc_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, data, `${functionName}()'s data`);
        return data;
    }
    async invokeRestWithCache(functionName, path, body, accessInfo, allowCache = false, ttlSeconds = 5) {
        if (allowCache) {
            const key = JSON.stringify({ functionName, path, body });
            const value = await this.cache.get(key);
            if (value) {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => 'Cached value');
                return value;
            }
            else {
                const data = await this.invokeRest(functionName, path, body, accessInfo);
                await this.cache.set(key, data, { ttl: ttlSeconds });
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.DEBUG, () => 'New value');
                return data;
            }
        }
        else {
            return this.invokeRest(functionName, path, body, accessInfo);
        }
    }
    async invokeRest(functionName, path, body, accessInfo) {
        let data = null;
        try {
            const { data: newData, errors } = await this.post(`/${path}`, {
                headers: {
                    authorization: accessInfo?.token
                },
                body
            });
            data = newData;
            if (lodash_1.default.isNil(data) && errors?.length > 0)
                throw new graphql_1.GraphQLError(errors[0].message, { extensions: errors[0].extensions });
        }
        catch (error) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.EXCEPTION, () => `invokeRest error:${JSON.stringify(error, null, 2)}`);
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.ErrorCodes.IntercallError, error?.extensions?.response?.body?.errors?.[0].message || error.message);
        }
        misc_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, data, `${functionName}()'s data`);
        return data;
    }
    async invokeExternalRestPost(functionName, path, body, headers) {
        let data = null;
        try {
            if (headers) {
                data = await this.post(`/${path}`, {
                    headers: { ...headers },
                    body
                });
            }
            else {
                data = await this.post(`/${path}`, {
                    body
                });
            }
        }
        catch (error) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.EXCEPTION, () => `invokeExternalRestPost error:${JSON.stringify(error, null, 2)}`);
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.ErrorCodes.IntercallError, error?.extensions?.response?.body?.errors?.[0].message || error.message);
        }
        misc_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, data, `${functionName}()'s data`);
        return data;
    }
    async invokeExternalRestGet(functionName, path, headers) {
        let data = null;
        try {
            if (headers) {
                data = await this.get(`/${path}`, { headers: { ...headers } });
            }
            else {
                data = await this.get(`/${path}`);
            }
        }
        catch (error) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.EXCEPTION, () => `invokeExternalRestGet error:${JSON.stringify(error, null, 2)}`);
            (0, cm_libs_context_aware_error_1.throwError)(cm_libs_context_aware_error_1.ErrorCodes.IntercallError, error?.extensions?.response?.body?.errors?.[0].message || error.message);
        }
        misc_1.SecureLog.log(cm_libs_context_aware_logger_1.Level.DEBUG, data, `${functionName}()'s data`);
        return data;
    }
}
exports.BaseRESTDataSource = BaseRESTDataSource;
//# sourceMappingURL=base.data-source.js.map